@@ -5,28 +5,37 @@
 	--------------------------------------------------------------------
 	SPDX-FileCopyrightText: 2006-2008 Tilman Benkert <thzs@gmx.net>
 	SPDX-FileCopyrightText: 2006-2009 Knut Franke <knut.franke@gmx.de>
-	SPDX-FileCopyrightText: 2012-2021 Alexander Semke <alexander.semke@web.de>
+	SPDX-FileCopyrightText: 2012-2023 Alexander Semke <alexander.semke@web.de>
 	SPDX-FileCopyrightText: 2017-2020 Stefan Gerlach <stefan.gerlach@uni.kn>
 
 	SPDX-License-Identifier: GPL-2.0-or-later
 */
 #include "Spreadsheet.h"
 #include "SpreadsheetModel.h"
+#include "SpreadsheetPrivate.h"
+#include "StatisticsSpreadsheet.h"
 #include "backend/core/AbstractAspect.h"
 #include "backend/core/AspectPrivate.h"
 #include "backend/core/column/ColumnStringIO.h"
 #include "backend/core/datatypes/DateTime2StringFilter.h"
 #include "backend/lib/XmlStreamReader.h"
+#include "backend/lib/commandtemplates.h"
 #include "backend/lib/macros.h"
 #include "backend/lib/trace.h"
 #include "backend/worksheet/plots/cartesian/CartesianPlot.h"
 #include "commonfrontend/spreadsheet/SpreadsheetView.h"
 
-#include <QIcon>
+#include "backend/lib/commandtemplates.h"
 
+#include <KConfig>
 #include <KConfigGroup>
 #include <KLocalizedString>
-#include <KSharedConfig>
+
+#include <QIcon>
+#include <QMenu>
+#include <QTimer>
+#include <QUndoCommand>
+#include <QXmlStreamWriter>
 
 #include <algorithm>
 
@@ -52,13 +61,15 @@
 */
 
 Spreadsheet::Spreadsheet(const QString& name, bool loading, AspectType type)
-	: AbstractDataSource(name, type) {
+	: AbstractDataSource(name, type)
+	, d_ptr(new SpreadsheetPrivate(this)) {
 	if (!loading)
 		init();
 }
 
 Spreadsheet::~Spreadsheet() {
 	delete m_model;
+	delete d_ptr;
 }
 
 /*!
@@ -94,9 +105,19 @@ SpreadsheetModel* Spreadsheet::model() const {
 QWidget* Spreadsheet::view() const {
 #ifndef SDK
 	if (!m_partView) {
-		bool readOnly = (this->parentAspect()->type() == AspectType::DatapickerCurve);
+		auto type = this->parentAspect()->type();
+		bool readOnly = (type == AspectType::Spreadsheet || type == AspectType::DatapickerCurve);
 		m_view = new SpreadsheetView(const_cast<Spreadsheet*>(this), readOnly);
 		m_partView = m_view;
+		connect(this, &Spreadsheet::viewAboutToBeDeleted, [this]() {
+			m_view = nullptr;
+		});
+
+		// navigate to the first cell and set the focus so the user can start directly entering new data
+		QTimer::singleShot(0, this, [=]() {
+			m_view->goToCell(0, 0);
+			m_view->setFocus();
+		});
 	}
 	return m_partView;
 #else
@@ -128,23 +149,30 @@ bool Spreadsheet::printPreview() const {
 #endif
 }
 
+StatisticsSpreadsheet* Spreadsheet::statisticsSpreadsheet() const {
+	Q_D(const Spreadsheet);
+	return d->statisticsSpreadsheet;
+}
+
 /*!
  * \brief Called when the application settings were changed.
  *  adjusts the appearence of the spreadsheet header.
  */
 void Spreadsheet::updateHorizontalHeader() {
 #ifndef SDK
-	const QString& oldHeader = m_model->headerData(0, Qt::Horizontal, Qt::DisplayRole).toString();
-	m_model->updateHorizontalHeader();
-	const QString& newHeader = m_model->headerData(0, Qt::Horizontal, Qt::DisplayRole).toString();
+	if (m_model) {
+		const QString& oldHeader = m_model->headerData(0, Qt::Horizontal, Qt::DisplayRole).toString();
+		m_model->updateHorizontalHeader();
+		const QString& newHeader = m_model->headerData(0, Qt::Horizontal, Qt::DisplayRole).toString();
 
-	// if the header name of the first column has changed (column mode to be shown, etc.),
-	// reset the column widths and request the view to adjuste the column sizes to the  content
-	if (oldHeader != newHeader && m_view) {
-		const auto& columns = children<Column>();
-		for (auto col : columns)
-			col->setWidth(0);
-		m_view->resizeHeader();
+		// if the header name of the first column has changed (column mode to be shown, etc.),
+		// reset the column widths and request the view to adjuste the column sizes to the  content
+		if (oldHeader != newHeader && m_view) {
+			const auto& columns = children<Column>();
+			for (auto col : columns)
+				col->setWidth(0);
+			m_view->resizeHeader();
+		}
 	}
 #endif
 }
@@ -167,26 +195,84 @@ int Spreadsheet::rowCount() const {
 	return result;
 }
 
-void Spreadsheet::removeRows(int first, int count) {
+class SpreadsheetSetRowsCountCmd : public QUndoCommand {
+public:
+	SpreadsheetSetRowsCountCmd(Spreadsheet* spreadsheet, bool insert, int first, int count, QUndoCommand* parent)
+		: QUndoCommand(parent)
+		, m_spreadsheet(spreadsheet)
+		, m_insert(insert)
+		, m_first(first)
+		, m_last(first + count - 1) {
+		if (insert)
+			setText(i18np("%1: insert 1 row", "%1: insert %2 rows", spreadsheet->name(), count));
+		else
+			setText(i18np("%1: remove 1 row", "%1: remove %2 rows", spreadsheet->name(), count));
+	}
+
+	virtual void redo() override {
+		WAIT_CURSOR;
+		if (m_insert)
+			Q_EMIT m_spreadsheet->rowsAboutToBeInserted(m_first, m_last);
+		else
+			Q_EMIT m_spreadsheet->rowsAboutToBeRemoved(m_first, m_last);
+
+		QUndoCommand::redo();
+
+		if (m_insert)
+			Q_EMIT m_spreadsheet->rowsInserted(m_spreadsheet->rowCount());
+		else
+			Q_EMIT m_spreadsheet->rowsRemoved(m_spreadsheet->rowCount());
+		RESET_CURSOR;
+		m_spreadsheet->emitRowCountChanged();
+	}
+
+	virtual void undo() override {
+		WAIT_CURSOR;
+		if (m_insert)
+			Q_EMIT m_spreadsheet->rowsAboutToBeRemoved(m_first, m_last);
+		else
+			Q_EMIT m_spreadsheet->rowsAboutToBeInserted(m_first, m_last);
+		QUndoCommand::undo();
+
+		if (m_insert)
+			Q_EMIT m_spreadsheet->rowsRemoved(m_spreadsheet->rowCount());
+		else
+			Q_EMIT m_spreadsheet->rowsInserted(m_spreadsheet->rowCount());
+		RESET_CURSOR;
+		m_spreadsheet->emitRowCountChanged();
+	}
+
+private:
+	Spreadsheet* m_spreadsheet;
+	bool m_insert;
+	int m_first;
+	int m_last;
+};
+
+void Spreadsheet::removeRows(int first, int count, QUndoCommand* parent) {
 	if (count < 1 || first < 0 || first + count > rowCount())
 		return;
-	WAIT_CURSOR;
-	beginMacro(i18np("%1: remove 1 row", "%1: remove %2 rows", name(), count));
+
+	auto* command = new SpreadsheetSetRowsCountCmd(this, false, first, count, parent);
+
 	for (auto* col : children<Column>())
-		col->removeRows(first, count);
-	endMacro();
-	RESET_CURSOR;
+		col->removeRows(first, count, command);
+
+	if (!parent)
+		exec(command);
 }
 
-void Spreadsheet::insertRows(int before, int count) {
+void Spreadsheet::insertRows(int before, int count, QUndoCommand* parent) {
 	if (count < 1 || before < 0 || before > rowCount())
 		return;
-	WAIT_CURSOR;
-	beginMacro(i18np("%1: insert 1 row", "%1: insert %2 rows", name(), count));
+
+	auto* command = new SpreadsheetSetRowsCountCmd(this, true, before, count, parent);
+
 	for (auto* col : children<Column>())
-		col->insertRows(before, count);
-	endMacro();
-	RESET_CURSOR;
+		col->insertRows(before, count, command);
+
+	if (!parent)
+		exec(command);
 }
 
 void Spreadsheet::appendRows(int count) {
@@ -197,6 +283,63 @@ void Spreadsheet::appendRow() {
 	insertRows(rowCount(), 1);
 }
 
+/*!
+ * removes all rows in the spreadsheet if the value in one of the columns is missing/empty.
+ */
+void Spreadsheet::removeEmptyRows() {
+	const auto& rows = rowsWithMissingValues();
+	if (rows.isEmpty())
+		return;
+
+	WAIT_CURSOR;
+	beginMacro(i18n("%1: remove rows with missing values", name()));
+
+	for (int row = rows.count() - 1; row >= 0; --row)
+		removeRows(rows.at(row), 1);
+
+	endMacro();
+	RESET_CURSOR;
+}
+
+/*!
+ * masks all rows in the spreadsheet if the value in one of the columns is missing/empty.
+ */
+void Spreadsheet::maskEmptyRows() {
+	const auto& rows = rowsWithMissingValues();
+	if (rows.isEmpty())
+		return;
+
+	WAIT_CURSOR;
+	beginMacro(i18n("%1: mask rows with missing values", name()));
+
+	const auto& columns = children<Column>();
+	for (int row : rows) {
+		for (const auto& col : columns)
+			col->setMasked(row);
+	}
+
+	endMacro();
+	RESET_CURSOR;
+}
+
+/*!
+ * returns the list of all rows having at least one missing/empty value.
+ */
+QVector<int> Spreadsheet::rowsWithMissingValues() const {
+	QVector<int> rows;
+	const auto& columns = children<Column>();
+	for (int row = 0; row < rowCount(); ++row) {
+		for (const auto& col : columns) {
+			if (col->asStringColumn()->textAt(row).isEmpty()) {
+				rows << row;
+				break;
+			}
+		}
+	}
+
+	return rows;
+}
+
 void Spreadsheet::appendColumns(int count) {
 	insertColumns(columnCount(), count);
 }
@@ -212,12 +355,104 @@ void Spreadsheet::prependColumns(int count) {
 /*!
   Sets the number of rows of the spreadsheet to \c new_size
 */
-void Spreadsheet::setRowCount(int new_size) {
+void Spreadsheet::setRowCount(int new_size, QUndoCommand* parent) {
 	int current_size = rowCount();
 	if (new_size > current_size)
-		insertRows(current_size, new_size - current_size);
+		insertRows(current_size, new_size - current_size, parent);
 	if (new_size < current_size && new_size >= 0)
-		removeRows(new_size, current_size - new_size);
+		removeRows(new_size, current_size - new_size, parent);
+}
+
+void Spreadsheet::initConnectionsLinking(const Spreadsheet* sender, const Spreadsheet* receiver) {
+	QObject::connect(sender, &Spreadsheet::aspectAboutToBeRemoved, receiver, &Spreadsheet::linkedSpreadsheetDeleted);
+	QObject::connect(sender, &Spreadsheet::rowCountChanged, receiver, &Spreadsheet::linkedSpreadsheetNewRowCount);
+}
+
+class SpreadsheetSetLinkingCmd : public QUndoCommand {
+public:
+	SpreadsheetSetLinkingCmd(Spreadsheet::Private* target,
+							 const Spreadsheet::Linking& newValue,
+							 const KLocalizedString& description,
+							 QUndoCommand* parent = nullptr)
+		: QUndoCommand(parent)
+		, m_target(target)
+		, m_linking(newValue) {
+		setText(description.subs(m_target->name()).toString());
+	}
+
+	void execute() {
+		if (m_target->linking.linkedSpreadsheet)
+			QObject::disconnect(m_target->linking.linkedSpreadsheet, nullptr, m_target->q, nullptr);
+
+		if (m_linking.linkedSpreadsheet) {
+			m_linking.linkedSpreadsheetPath = m_linking.linkedSpreadsheet->path();
+			m_target->q->initConnectionsLinking(m_linking.linkedSpreadsheet, m_target->q);
+		}
+
+		const Spreadsheet::Linking l = m_target->linking;
+		m_target->linking = m_linking;
+		m_linking = l;
+	}
+
+	virtual void redo() override {
+		execute();
+		QUndoCommand::redo();
+		finalize();
+	}
+
+	virtual void undo() override {
+		execute();
+		QUndoCommand::undo();
+		finalize();
+	}
+
+	void finalize() const {
+		Q_EMIT m_target->q->linkingChanged(m_target->linking.linking);
+		Q_EMIT m_target->q->linkedSpreadsheetChanged(m_target->linking.linkedSpreadsheet);
+	}
+
+private:
+	Spreadsheet::Private* m_target;
+	Spreadsheet::Linking m_linking;
+};
+
+BASIC_SHARED_D_READER_IMPL(Spreadsheet, bool, linking, linking.linking)
+void Spreadsheet::setLinking(bool linking) {
+	Q_D(Spreadsheet);
+	if (linking != d->linking.linking) {
+		Linking l = d->linking;
+		l.linking = linking;
+		auto parent = new SpreadsheetSetLinkingCmd(d, l, ki18n("%1: set linking"));
+		if (linking && d->linking.linkedSpreadsheet)
+			setRowCount(d->linking.linkedSpreadsheet->rowCount(), parent);
+		exec(parent);
+	}
+}
+
+BASIC_SHARED_D_READER_IMPL(Spreadsheet, const Spreadsheet*, linkedSpreadsheet, linking.linkedSpreadsheet)
+void Spreadsheet::setLinkedSpreadsheet(const Spreadsheet* linkedSpreadsheet, bool skipUndo) {
+	Q_D(Spreadsheet);
+	if (!d->linking.linking)
+		return; // Do not allow setting a spreadsheet when linking is disabled
+
+	if (linkedSpreadsheet != d->linking.linkedSpreadsheet) {
+		if (skipUndo) {
+			d->linking.linkedSpreadsheet = linkedSpreadsheet;
+			initConnectionsLinking(linkedSpreadsheet, this);
+		} else {
+			Linking l = d->linking;
+			l.linkedSpreadsheet = linkedSpreadsheet;
+			auto* parent = new SpreadsheetSetLinkingCmd(d, l, ki18n("%1: set linked spreadsheet"));
+			if (d->linking.linking && linkedSpreadsheet)
+				setRowCount(linkedSpreadsheet->rowCount(), parent);
+			exec(parent);
+		}
+	}
+}
+
+QString Spreadsheet::linkedSpreadsheetPath() const {
+	Q_D(const Spreadsheet);
+	return d->linking.spreadsheetPath();
 }
 
 /*!
@@ -253,55 +488,111 @@ int Spreadsheet::columnCount(AbstractColumn::PlotDesignation pd) const {
 	return count;
 }
 
-void Spreadsheet::removeColumns(int first, int count) {
+class SpreadsheetSetColumnsCountCmd : public QUndoCommand {
+public:
+	SpreadsheetSetColumnsCountCmd(Spreadsheet* spreadsheet, bool insert, int first, int count, QUndoCommand* parent)
+		: QUndoCommand(parent)
+		, m_spreadsheet(spreadsheet)
+		, m_insert(insert)
+		, m_first(first)
+		, m_last(first + count - 1) {
+		if (insert)
+			setText(i18np("%1: insert 1 column", "%1: insert %2 columns", spreadsheet->name(), count));
+		else
+			setText(i18np("%1: remove 1 column", "%1: remove %2 columns", spreadsheet->name(), count));
+	}
+
+	virtual void redo() override {
+		WAIT_CURSOR;
+		if (m_insert)
+			Q_EMIT m_spreadsheet->aspectsAboutToBeInserted(m_first, m_last);
+		else
+			Q_EMIT m_spreadsheet->aspectsAboutToBeRemoved(m_first, m_last);
+
+		QUndoCommand::redo();
+
+		if (m_insert)
+			Q_EMIT m_spreadsheet->aspectsInserted(m_first, m_last);
+		else
+			Q_EMIT m_spreadsheet->aspectsRemoved();
+		RESET_CURSOR;
+		m_spreadsheet->emitColumnCountChanged();
+	}
+
+	virtual void undo() override {
+		WAIT_CURSOR;
+		if (m_insert)
+			Q_EMIT m_spreadsheet->aspectsAboutToBeRemoved(m_first, m_last);
+		else
+			Q_EMIT m_spreadsheet->aspectsAboutToBeInserted(m_first, m_last);
+		QUndoCommand::undo();
+
+		if (m_insert)
+			Q_EMIT m_spreadsheet->aspectsRemoved();
+		else
+			Q_EMIT m_spreadsheet->aspectsInserted(m_first, m_last);
+		RESET_CURSOR;
+		m_spreadsheet->emitColumnCountChanged();
+	}
+
+private:
+	Spreadsheet* m_spreadsheet;
+	bool m_insert;
+	int m_first;
+	int m_last;
+};
+
+void Spreadsheet::removeColumns(int first, int count, QUndoCommand* parent) {
 	if (count < 1 || first < 0 || first + count > columnCount())
 		return;
-	WAIT_CURSOR;
-	beginMacro(i18np("%1: remove 1 column", "%1: remove %2 columns", name(), count));
-	for (int i = 0; i < count; i++)
-		child<Column>(first)->remove();
-	endMacro();
-	RESET_CURSOR;
+
+	auto* command = new SpreadsheetSetColumnsCountCmd(this, false, first, count, parent);
+	bool execute = false;
+	if (!parent) {
+		execute = true;
+		parent = command;
+	}
+
+	const auto& columns = children<Column>();
+	for (int i = (first + count - 1); i >= first; i--)
+		columns.at(i)->remove(parent);
+
+	if (execute)
+		exec(command);
 }
 
-void Spreadsheet::insertColumns(int before, int count) {
-	WAIT_CURSOR;
-	beginMacro(i18np("%1: insert 1 column", "%1: insert %2 columns", name(), count));
-	auto* before_col = column(before);
+void Spreadsheet::insertColumns(int before, int count, QUndoCommand* parent) {
+	auto* command = new SpreadsheetSetColumnsCountCmd(this, true, before, count, parent);
+	bool execute = false;
+	if (!parent) {
+		execute = true;
+		parent = command;
+	}
 	const int cols = columnCount();
 	const int rows = rowCount();
 	for (int i = 0; i < count; i++) {
 		auto* new_col = new Column(QString::number(cols + i + 1), AbstractColumn::ColumnMode::Double);
 		new_col->setPlotDesignation(AbstractColumn::PlotDesignation::Y);
 		new_col->insertRows(0, rows);
-		insertChildBefore(new_col, before_col);
+		insertChild(new_col, before + i, parent);
 	}
-	endMacro();
-	RESET_CURSOR;
+
+	if (execute)
+		exec(command);
 }
 
 /*!
   Sets the number of columns to \c new_size
 */
-void Spreadsheet::setColumnCount(int new_size) {
+void Spreadsheet::setColumnCount(int new_size, QUndoCommand* parent) {
 	int old_size = columnCount();
 	if (old_size == new_size || new_size < 0)
 		return;
 
-	// suppress handling of child add and remove signals when adding/removing multiple columns
-	// TODO: undo/redo of this step is still very slow for a big number of columns
-	disconnect(this, &Spreadsheet::aspectAdded, m_view, &SpreadsheetView::handleAspectAdded);
-	if (m_model)
-		m_model->suppressSignals(true);
-
 	if (new_size < old_size)
-		removeColumns(new_size, old_size - new_size);
+		removeColumns(new_size, old_size - new_size, parent);
 	else
-		insertColumns(old_size, new_size - old_size);
-
-	connect(this, &Spreadsheet::aspectAdded, m_view, &SpreadsheetView::handleAspectAdded);
-	if (m_model)
-		m_model->suppressSignals(false);
+		insertColumns(old_size, new_size - old_size, parent);
 }
 
 /*!
@@ -316,6 +607,26 @@ void Spreadsheet::clear() {
 	RESET_CURSOR;
 }
 
+void Spreadsheet::clear(const QVector<Column*>& columns) {
+	auto* parent = new LongExecutionCmd(i18n("%1: clear selected columns", name()));
+
+	// 	if (formulaModeActive()) {
+	// 		for (auto* col : selectedColumns()) {
+	// 			col->setSuppressDataChangedSignal(true);
+	// 			col->clearFormulas();
+	// 			col->setSuppressDataChangedSignal(false);
+	// 			col->setChanged();
+	// 		}
+	// 	} else {
+	for (auto* col : columns) {
+		col->setSuppressDataChangedSignal(true);
+		col->clear(parent);
+		col->setSuppressDataChangedSignal(false);
+		col->setChanged();
+	}
+	exec(parent);
+}
+
 /*!
   Clears all mask in the spreadsheet.
 */
@@ -334,48 +645,39 @@ void Spreadsheet::clearMasks() {
 QMenu* Spreadsheet::createContextMenu() {
 	QMenu* menu = AbstractPart::createContextMenu();
 	Q_ASSERT(menu);
-	Q_EMIT requestProjectContextMenu(menu);
+	if (type() != AspectType::StatisticsSpreadsheet)
+		Q_EMIT requestProjectContextMenu(menu);
+	else {
+		menu->addSeparator();
+		auto* action = new QAction(QIcon::fromTheme(QLatin1String("edit-delete")), i18n("Delete"), this);
+		connect(action, &QAction::triggered, this, [=]() {
+			auto* parentSpreadsheet = static_cast<Spreadsheet*>(parentAspect());
+			parentSpreadsheet->toggleStatisticsSpreadsheet(false);
+		});
+		menu->addAction(action);
+	}
 	return menu;
 }
 
 void Spreadsheet::fillColumnContextMenu(QMenu* menu, Column* column) {
+#ifndef SDK
 	if (m_view)
 		m_view->fillColumnContextMenu(menu, column);
+#else
+	Q_UNUSED(menu)
+	Q_UNUSED(column)
+#endif
 }
 
 void Spreadsheet::moveColumn(int from, int to) {
-	Column* col = child<Column>(from);
+	const auto& columns = children<Column>();
+	auto* col = columns.at(from);
 	beginMacro(i18n("%1: move column %2 from position %3 to %4.", name(), col->name(), from + 1, to + 1));
 	col->remove();
-	insertChildBefore(col, child<Column>(to));
+	insertChildBefore(col, columns.at(to));
 	endMacro();
 }
 
-void Spreadsheet::copy(Spreadsheet* other) {
-	WAIT_CURSOR;
-	beginMacro(i18n("%1: copy %2", name(), other->name()));
-
-	for (auto* col : children<Column>())
-		col->remove();
-	for (auto* src_col : other->children<Column>()) {
-		Column* new_col = new Column(src_col->name(), src_col->columnMode());
-		new_col->copy(src_col);
-		new_col->setPlotDesignation(src_col->plotDesignation());
-		QVector<Interval<int>> masks = src_col->maskedIntervals();
-		for (const auto& iv : masks)
-			new_col->setMasked(iv);
-		QVector<Interval<int>> formulas = src_col->formulaIntervals();
-		for (const auto& iv : formulas)
-			new_col->setFormula(iv, src_col->formula(iv.start()));
-		new_col->setWidth(src_col->width());
-		addChild(new_col);
-	}
-	setComment(other->comment());
-
-	endMacro();
-	RESET_CURSOR;
-}
-
 // FIXME: replace index-based API with Column*-based one
 /*!
   Determines the corresponding X column.
@@ -831,6 +1133,17 @@ void Spreadsheet::childDeselected(const AbstractAspect* aspect) {
 	}
 }
 
+void Spreadsheet::linkedSpreadsheetDeleted() {
+	Q_D(Spreadsheet);
+	Linking l = d->linking;
+	l.linkedSpreadsheet = nullptr;
+	exec(new SpreadsheetSetLinkingCmd(d, l, ki18n("%1: linked spreadsheet removed")));
+}
+
+void Spreadsheet::linkedSpreadsheetNewRowCount(int rowCount) {
+	setRowCount(rowCount);
+}
+
 /*!
  *  Emits the signal to select or to deselect the column number \c index in the project explorer,
  *  if \c selected=true or \c selected=false, respectively.
@@ -864,22 +1177,51 @@ QVector<AspectType> Spreadsheet::dropableOn() const {
 	return vec;
 }
 
-//##############################################################################
-//##################  Serialization/Deserialization  ###########################
-//##############################################################################
+void Spreadsheet::toggleStatisticsSpreadsheet(bool on) {
+	Q_D(Spreadsheet);
+	if (on) {
+		if (d->statisticsSpreadsheet)
+			return;
+
+		d->statisticsSpreadsheet = new StatisticsSpreadsheet(this);
+		addChildFast(d->statisticsSpreadsheet);
+	} else {
+		if (!d->statisticsSpreadsheet)
+			return;
+
+		setUndoAware(false);
+		removeChild(d->statisticsSpreadsheet);
+		setUndoAware(true);
+		d->statisticsSpreadsheet = nullptr;
+	}
+}
+
+// ##############################################################################
+// ##################  Serialization/Deserialization  ###########################
+// ##############################################################################
 /*!
   Saves as XML.
  */
 void Spreadsheet::save(QXmlStreamWriter* writer) const {
+	Q_D(const Spreadsheet);
 	writer->writeStartElement(QStringLiteral("spreadsheet"));
 	writeBasicAttributes(writer);
 	writeCommentElement(writer);
 
+	writer->writeStartElement(QLatin1String("linking"));
+	writer->writeAttribute(QStringLiteral("enabled"), QString::number(d->linking.linking));
+	writer->writeAttribute(QStringLiteral("spreadsheet"), d->linking.spreadsheetPath());
+	writer->writeEndElement();
+
 	// columns
 	const auto& columns = children<Column>(ChildIndexFlag::IncludeHidden);
 	for (auto* column : columns)
 		column->save(writer);
 
+	// statistics spreadsheet, if available
+	if (d->statisticsSpreadsheet)
+		d->statisticsSpreadsheet->save(writer);
+
 	writer->writeEndElement(); // "spreadsheet"
 }
 
@@ -887,20 +1229,34 @@ void Spreadsheet::save(QXmlStreamWriter* writer) const {
   Loads from XML.
 */
 bool Spreadsheet::load(XmlStreamReader* reader, bool preview) {
+	Q_D(Spreadsheet);
 	if (!readBasicAttributes(reader))
 		return false;
 
+	QString str;
+	QXmlStreamAttributes attribs;
+
 	// read child elements
 	while (!reader->atEnd()) {
 		reader->readNext();
 
-		if (reader->isEndElement())
+		if (reader->isEndElement() && reader->name() == QLatin1String("spreadsheet"))
 			break;
 
 		if (reader->isStartElement()) {
 			if (reader->name() == QLatin1String("comment")) {
 				if (!readCommentElement(reader))
 					return false;
+			} else if (reader->name() == QLatin1String("linking")) {
+				attribs = reader->attributes();
+				str = attribs.value(QStringLiteral("enabled")).toString();
+				if (str.isEmpty())
+					reader->raiseMissingAttributeWarning(QStringLiteral("enabled"));
+				else
+					d->linking.linking = static_cast<bool>(str.toInt());
+
+				str = attribs.value(QStringLiteral("spreadsheet")).toString();
+				d->linking.linkedSpreadsheetPath = str;
 			} else if (reader->name() == QLatin1String("column")) {
 				Column* column = new Column(QString());
 				column->setIsLoading(true);
@@ -910,8 +1266,15 @@ bool Spreadsheet::load(XmlStreamReader* reader, bool preview) {
 					return false;
 				}
 				addChildFast(column);
+			} else if (reader->name() == QLatin1String("statisticsSpreadsheet")) {
+				d->statisticsSpreadsheet = new StatisticsSpreadsheet(this, true);
+				if (!d->statisticsSpreadsheet->load(reader, preview)) {
+					delete d->statisticsSpreadsheet;
+					d->statisticsSpreadsheet = nullptr;
+				} else
+					addChildFast(d->statisticsSpreadsheet);
 			} else { // unknown element
-				reader->raiseWarning(i18n("unknown element '%1'", reader->name().toString()));
+				reader->raiseUnknownElementWarning();
 				if (!reader->skipToEndElement())
 					return false;
 			}
@@ -921,19 +1284,22 @@ bool Spreadsheet::load(XmlStreamReader* reader, bool preview) {
 	return !reader->hasError();
 }
 
-//##############################################################################
-//########################  Data Import  #######################################
-//##############################################################################
+// ##############################################################################
+// ########################  Data Import  #######################################
+// ##############################################################################
 int Spreadsheet::prepareImport(std::vector<void*>& dataContainer,
 							   AbstractFileFilter::ImportMode importMode,
 							   int actualRows,
 							   int actualCols,
-							   QStringList colNameList,
-							   QVector<AbstractColumn::ColumnMode> columnMode,
+							   const QStringList& colNameList,
+							   const QVector<AbstractColumn::ColumnMode>& columnMode,
+							   bool& ok,
 							   bool initializeContainer) {
+	Q_D(Spreadsheet);
 	PERFTRACE(QLatin1String(Q_FUNC_INFO));
 	DEBUG(Q_FUNC_INFO << ", resize spreadsheet to rows = " << actualRows << " and cols = " << actualCols)
 	QDEBUG(Q_FUNC_INFO << ", column name list = " << colNameList)
+	assert(d->m_usedInPlots.size() == 0);
 	int columnOffset = 0;
 	setUndoAware(false);
 	if (m_model != nullptr)
@@ -941,30 +1307,41 @@ int Spreadsheet::prepareImport(std::vector<void*>& dataContainer,
 
 	// make the available columns undo unaware before we resize and rename them below,
 	// the same will be done for new columns in this->resize().
-	for (int i = 0; i < childCount<Column>(); i++)
-		child<Column>(i)->setUndoAware(false);
+	{
+		const auto& columns = children<Column>();
+		for (auto* column : qAsConst(columns))
+			column->setUndoAware(false);
+	}
 
 	columnOffset = this->resize(importMode, colNameList, actualCols);
+	if (initializeContainer)
+		dataContainer.resize(actualCols);
+	const auto& columns = children<Column>(); // Get new children because of the resize it might be different
 
 	// resize the spreadsheet
-	if (importMode == AbstractFileFilter::ImportMode::Replace) {
-		clear();
-		setRowCount(actualRows);
-	} else {
-		if (rowCount() < actualRows)
-			setRowCount(actualRows);
+	if (initializeContainer) {
+		try {
+			if (importMode == AbstractFileFilter::ImportMode::Replace) {
+				clear();
+				setRowCount(actualRows);
+			} else {
+				if (rowCount() < actualRows)
+					setRowCount(actualRows);
+			}
+		} catch (std::bad_alloc&) {
+			ok = false;
+			return 0;
+		}
 	}
 
 	if (columnMode.size() < actualCols) {
-		DEBUG(Q_FUNC_INFO << ", columnMode[] size is too small! Giving up.");
+		QDEBUG(Q_FUNC_INFO << ", columnMode[] size " << columnMode.size() << " is too small, should be " << actualCols << "! Giving up.");
 		return -1;
 	}
 
-	if (initializeContainer)
-		dataContainer.resize(actualCols);
 	for (int n = 0; n < actualCols; n++) {
 		// data() returns a void* which is a pointer to any data type (see ColumnPrivate.cpp)
-		Column* column = this->child<Column>(columnOffset + n);
+		auto* column = columns.at(columnOffset + n);
 		DEBUG(" column " << n << " columnMode = " << ENUM_TO_STRING(AbstractColumn, ColumnMode, columnMode[n]));
 		column->setColumnModeFast(columnMode[n]);
 
@@ -1008,6 +1385,7 @@ int Spreadsheet::prepareImport(std::vector<void*>& dataContainer,
 			}
 			}
 		} else {
+			// Assign already allocated datacontainer to the column
 			column->setData(dataContainer[n]);
 		}
 	}
@@ -1015,6 +1393,7 @@ int Spreadsheet::prepareImport(std::vector<void*>& dataContainer,
 
 	// DEBUG(Q_FUNC_INFO << ", DONE");
 
+	ok = true;
 	return columnOffset;
 }
 
@@ -1022,12 +1401,12 @@ int Spreadsheet::prepareImport(std::vector<void*>& dataContainer,
 	resize data source to cols columns
 	returns column offset depending on import mode
 */
-int Spreadsheet::resize(AbstractFileFilter::ImportMode mode, QStringList names, int cols) {
+int Spreadsheet::resize(AbstractFileFilter::ImportMode mode, const QStringList& names, int cols) {
 	//	PERFTRACE(Q_FUNC_INFO);
 	DEBUG(Q_FUNC_INFO << ", mode = " << ENUM_TO_STRING(AbstractFileFilter, ImportMode, mode) << ", cols = " << cols)
 	// QDEBUG("	column name list = " << colNameList)
-	//  name additional columns
-	emit aboutToResize();
+
+	Q_EMIT aboutToResize(); // call this to disable the retransforms in worksheet elements in Project
 
 	// make sure the column names provided by the user don't have any duplicates
 	QStringList uniqueNames;
@@ -1067,40 +1446,49 @@ int Spreadsheet::resize(AbstractFileFilter::ImportMode mode, QStringList names,
 		}
 	} else if (mode == AbstractFileFilter::ImportMode::Replace) {
 		// replace completely the previous content of the data source with the content to be imported.
-		int columns = childCount<Column>();
+		int columnsCount = childCount<Column>();
 
-		if (columns > cols) {
+		if (columnsCount > cols) {
 			// there are more columns in the data source than required -> remove the superfluous columns
-			for (int i = 0; i < columns - cols; i++)
+			for (int i = 0; i < columnsCount - cols; i++)
 				removeChild(child<Column>(0));
 		} else {
 			// create additional columns if needed
-			// disconnect from the handleAspectAdded slot in the view, no need to handle it when adding new columns during the import
-			disconnect(this, &Spreadsheet::aspectAdded, m_view, &SpreadsheetView::handleAspectAdded);
-			for (int i = columns; i < cols; i++) {
+			if (cols - columnsCount > 30)
+				Q_EMIT manyAspectsAboutToBeInserted();
+			Q_EMIT aspectsAboutToBeInserted(columnsCount, cols - 1);
+			for (int i = columnsCount; i < cols; i++) {
 				newColumn = new Column(uniqueNames.at(i), AbstractColumn::ColumnMode::Double);
 				newColumn->resizeTo(rows);
 				newColumn->setUndoAware(false);
 				newColumn->resizeTo(rows);
 				addChildFast(newColumn); // in the replace mode, we can skip checking the uniqueness of the names and use the "fast" method
 			}
-			connect(this, &Spreadsheet::aspectAdded, m_view, &SpreadsheetView::handleAspectAdded);
+			Q_EMIT aspectsInserted(columnsCount, cols - 1);
 		}
 
-		// 1. suppressretransform for all WorksheetElements
-		// 2. rename the columns that were already available
-		// 3. suppress the dataChanged signal for all columns
-		// 4. send aspectDescriptionChanged because otherwise the column
-		//    will not be connected again to the curves (project.cpp, descriptionChanged)
-		// 5. Enable retransform for all WorksheetElements
-		for (int i = 0; i < childCount<Column>(); i++) {
-			child<Column>(i)->setSuppressDataChangedSignal(true);
-			Q_EMIT child<Column>(i)->reset(child<Column>(i));
-			child<Column>(i)->setName(uniqueNames.at(i), AbstractAspect::NameHandling::UniqueNotRequired);
-			child<Column>(i)->aspectDescriptionChanged(child<Column>(i));
+		// 1. if the column name has changed, call Column::reset() to disconnect all dependent objects from the dataChanged signal
+		// 2. suppress the dataChanged signal for all columns (will be restored later in finalizeImport())
+		// 3. rename the columns that were already available
+		// 4. column->aspectDescriptionChanged() to trigger the update of the dependencies on column in Project.
+		const auto& columns = children<Column>();
+		int index = 0;
+		Q_D(Spreadsheet);
+		for (auto* column : columns) {
+			column->setSuppressDataChangedSignal(true);
+			const auto& newName = uniqueNames.at(index);
+			if (column->name() != newName) {
+				column->addUsedInPlots(d->m_usedInPlots);
+				column->reset();
+				column->setName(newName, AbstractAspect::NameHandling::UniqueNotRequired);
+				column->aspectDescriptionChanged(column);
+			}
+			++index;
 		}
 	}
-	emit resizeFinished();
+
+	Q_EMIT resizeFinished(); // call this to re-enable the retransforms in worksheet elements in Project
+
 	return columnOffset;
 }
 
@@ -1110,28 +1498,30 @@ void Spreadsheet::finalizeImport(size_t columnOffset,
 								 const QString& dateTimeFormat,
 								 AbstractFileFilter::ImportMode importMode) {
 	PERFTRACE(QLatin1String(Q_FUNC_INFO));
+	Q_D(Spreadsheet);
 	// DEBUG(Q_FUNC_INFO << ", start/end col = " << startColumn << " / " << endColumn);
 
 	// determine the dependent plots
-	QVector<CartesianPlot*> plots;
 	if (importMode == AbstractFileFilter::ImportMode::Replace) {
 		for (size_t n = startColumn; n <= endColumn; n++) {
-			Column* column = this->column((int)(columnOffset + n - startColumn));
+			auto* column = this->column((int)(columnOffset + n - startColumn));
 			if (column)
-				column->addUsedInPlots(plots);
+				column->addUsedInPlots(d->m_usedInPlots);
 		}
+	}
 
+	if (importMode == AbstractFileFilter::ImportMode::Replace) {
 		// suppress retransform in the dependent plots
-		for (auto* plot : plots)
+		for (auto* plot : d->m_usedInPlots)
 			plot->setSuppressRetransform(true);
 	}
 
 	// set the comments for each of the columns if datasource is a spreadsheet
 	const int rows = rowCount();
-	for (size_t n = startColumn; n <= endColumn; n++) {
-		// DEBUG(Q_FUNC_INFO << ", column " << columnOffset + n - startColumn);
-		Column* column = this->column((int)(columnOffset + n - startColumn));
-		// DEBUG(Q_FUNC_INFO << ", type " << static_cast<int>(column->columnMode()));
+	for (size_t col = startColumn; col <= endColumn; col++) {
+		// DEBUG(Q_FUNC_INFO << ", column " << columnOffset + col - startColumn);
+		Column* column = this->column((int)(columnOffset + col - startColumn));
+		DEBUG(Q_FUNC_INFO << ", type " << ENUM_TO_STRING(AbstractColumn, ColumnMode, column->columnMode()))
 
 		QString comment;
 		switch (column->columnMode()) {
@@ -1169,27 +1559,49 @@ void Spreadsheet::finalizeImport(size_t columnOffset,
 
 	if (importMode == AbstractFileFilter::ImportMode::Replace) {
 		// retransform the dependent plots
-		for (auto* plot : plots) {
+		for (auto* plot : d->m_usedInPlots) {
 			plot->setSuppressRetransform(false);
 			plot->dataChanged(-1, -1); // TODO: check if all ranges must be updated
 		}
 	}
+	d->m_usedInPlots.clear();
 
 	// make the spreadsheet and all its children undo aware again
 	setUndoAware(true);
 	for (int i = 0; i < childCount<Column>(); i++)
 		child<Column>(i)->setUndoAware(true);
 
-	if (m_model != nullptr)
+	if (m_model)
 		m_model->suppressSignals(false);
 
 #ifndef SDK
-	if (m_partView != nullptr && m_view != nullptr)
+	if (m_partView && m_view)
 		m_view->resizeHeader();
 #endif
+
 	// row count most probably changed after the import, notify the dock widget.
 	// no need to notify about the column count change, this is already done by add/removeChild signals
-	rowCountChanged(rowCount());
+	Q_EMIT rowCountChanged(rowCount());
 
 	// DEBUG(Q_FUNC_INFO << " DONE");
 }
+
+void Spreadsheet::handleAspectUpdated(const QString& aspectPath, const AbstractAspect* aspect) {
+	const auto* sh = dynamic_cast<const Spreadsheet*>(aspect);
+	if (sh && linkedSpreadsheetPath() == aspectPath) {
+		setUndoAware(false);
+		setLinkedSpreadsheet(sh);
+		setUndoAware(true);
+	}
+}
+
+// ##############################################################################
+// ######################### Private implementation #############################
+// ##############################################################################
+SpreadsheetPrivate::SpreadsheetPrivate(Spreadsheet* owner)
+	: q(owner) {
+}
+
+QString SpreadsheetPrivate::name() const {
+	return q->name();
+}
