@@ -24,11 +24,12 @@ public:
 	~ImportDatasetDialog() override;
 
 	QString selectedObject() const override;
-	void importToDataset(DatasetHandler*, QStatusBar*) const;
-	void importTo(QStatusBar*) const override;
+	bool importTo(QStatusBar*) const override;
 
 private:
+	MainWin* m_mainWin{nullptr};
 	ImportDatasetWidget* m_importDatasetWidget;
+	enum class Status { SUCCESS = 0, FAILURE = 1 };
 
 protected Q_SLOTS:
 	void checkOkButton() override;
