@@ -56,6 +56,17 @@ private Q_SLOTS:
 	void datapickerDateTime();
 
 	void datapickerDeleteCurvePoint();
+
+	void datapickerImageLoadImageAbsolute();
+	void datapickerImageLoadImageRelative();
+	void datapickerImageLoadImageEmbeddAbsolute();
+	void datapickerImageLoadImageEmbeddAbsoluteUndoRedo();
+	void datapickerImageLoadImageEmbeddRelative();
+	void datapickerImageLoadImageEmbeddRelativeUndoRedo();
+	void datapickerImageClipboard();
+	void datapickerImageClipboardSelectImageFromPath();
+
+	void saveLoad();
 };
 
 #endif // DATAPICKERTEST_H
